//
//  PlayerViewController.swift
//  AmpChromecastSample
//
//  Created by Lopez Valenciano, Miguel on 11/10/17.
//  Copyright © 2017 Akamai. All rights reserved.
//

import UIKit
import AmpCore
import AmpChromecast
import GoogleCast

//let url = "https://commondatastorage.googleapis.com/gtv-videos-bucket/CastVideos/mp4/DesigningForGoogleCast.mp4"
let url = "https://commondatastorage.googleapis.com/gtv-videos-bucket/CastVideos/hls/Sintel.m3u8"

class PlayerViewController: UIViewController {
    
    @IBOutlet weak var playerView: UIView!
    var ampPlayer: AmpPlayer!
    var ampChromecast: AmpChromecastManager!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.ampPlayer = AmpPlayer(parentView: self.playerView)
    self.ampPlayer.setLicense("AwH4ufhrEyYgvu5dFMNtHJntZpWMB9CA6Gb4sSAn5s2t9cLPSBgV5T5nUAtsOtH7K23SS4F2Tlxu5Bz5GJHxWZD6DIhLcbc+RdBk+75LFL2/twdH9vhGBAQsEWohZVxjfJq5TOY20EcWU3A3cQly1pDU7ENCdE6A+UzrQASsVPSOGbjcHD3v2QLRMVK1tjBpBHv7Efh+dSNFWKsSLfNhZlRI6664Al+Jr9TuzjwyrloZcPxfDXznkPXNIqxyRkmAyT4=")
        self.ampPlayer.autoplay = true
        self.ampPlayer.registerObserver(self)
        
        self.ampChromecast = AmpChromecastManager(ampPlayer: self.ampPlayer)
        self.ampChromecast.registerObserver(self)
        
        let queueItems = [createQueueItem(url: "http://multiplatform-f.akamaihd.net/i/multi/april11/sintel/sintel-hd_,512x288_450_b,640x360_700_b,768x432_1000_b,1024x576_1400_m,.mp4.csmil/master.m3u8"), createQueueItem(url: "http://multiformatlive-f.akamaihd.net/i/demostream_1@2131/master.m3u8"), createQueueItem(url: "http://fnctve-i.akamaihd.net/hls/live/200669/fnctve/master.m3u8?hdnea=st=1433954196~exp=1907308086~acl=/*~hmac=62bfdfc61282e7be4d551454fd988dfafb20ec780d8011cc5c058b63d9b6f25d"), createQueueItem(url: "http://hsn.mpl.miisolutions.net/hsn-live01/_definst_/smil:HSN1_ipad.smil/playlist.m3u8"), createQueueItem(url: "https://ownvodhd-vh.akamaihd.net/i/video/201512/sss/vod/20151209-sss-615-canfield-episode-vod.mp4/master.m3u8")]
        self.ampChromecast.sessionManager?.currentSession?.remoteMediaClient?.queueLoad(queueItems, start: 0, repeatMode: GCKMediaRepeatMode.all)
        
        self.ampPlayer.handleUrl(url)
    }
    
    func createQueueItem(url: String) -> GCKMediaQueueItem {
        let contenType = AmpUtils.urlContentType(url) ?? "application/x-mpegurl"
        let mediaInfo = self.createMediaInformation(url: url, contenType: contenType, duration: 20)
        return GCKMediaQueueItem(mediaInformation: mediaInfo, autoplay: true, startTime: 0, playbackDuration: 20, preloadTime: 5, activeTrackIDs: nil, customData: nil)
    }
    
    internal func createMediaInformation(
        url: String,
        contenType: String,
        duration: Double,
        metadata: GCKMediaMetadata? = nil,
        mediaTracks: [GCKMediaTrack] = [],
        textTrackStyle: GCKMediaTextTrackStyle? = nil,
        customData: [AnyObject]? = nil) -> GCKMediaInformation {
        
        AmpLogger.info("createMediaInformation: url:\(url) contentType:\(contenType)")
        
        let mediaInformation = GCKMediaInformation(
            contentID: url,
            // TODO: They always use .None, but is this okay?
            streamType: GCKMediaStreamType.buffered,
            contentType: contenType,
            metadata: metadata,
            // TODO: Same issue as streamType
            streamDuration: duration,
            mediaTracks: mediaTracks,
            textTrackStyle: textTrackStyle,
            customData: customData
        )
        
        return mediaInformation
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    override open func viewWillDisappear(_ animated:Bool) {
        if (self.isMovingFromParentViewController) {
            self.ampChromecast?.disconnect()
            self.ampPlayer.stop()
            self.ampPlayer = nil
            print("Stopped")
        }
        super.viewWillDisappear(animated)
    }
    
    
}

extension PlayerViewController : PlayerEventObserver {
    
    func onCuepointReached(_ ampPlayer: AmpPlayer, atTime: Double) {
        AmpLogger.info("Cuepoint at \(atTime)")
    }
    
    func onExitFullscreen(_ ampPlayer: AmpPlayer) {
        print("onExitFullscreen")
    }
}

extension PlayerViewController : AmpChromecastObserver {
    
    func onChromecastPlaybackStateChanged(_ manager: AmpChromecastManager) {
        print("onChromecastPlaybackStateChanged")
    }
    
    func onChromecastError(_ manager: AmpChromecastManager, error: Error?) {
        print("onChromecastError \(error)")
    }
    
}
